/// @file   AuthnOracle.h
/// @brief  Oracle UserGroupDB Implementation.
/// @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
#ifndef AUTHNORACLE_H
#define AUTHNORACLE_H

#include <dmlite/cpp/authn.h>
#include <occi.h>

namespace dmlite {
  
  class AuthnOracle: public Authn {
   public:
    /// Constructor.
    AuthnOracle(oracle::occi::ConnectionPool* pool,
                oracle::occi::Connection* conn,
                const std::string& mapfile,
                bool hostDnIsRoot) throw(DmException);

    /// Destructor.
    ~AuthnOracle() throw(DmException);

    // Override

    std::string getImplId(void) const throw();

    SecurityContext* createSecurityContext(const SecurityCredentials& cred) throw (DmException);

    GroupInfo newGroup   (const std::string& gname) throw (DmException);
    GroupInfo getGroup   (const std::string& groupName) throw (DmException);
    GroupInfo getGroup   (const std::string& key,
                          const boost::any& value) throw (DmException);
    void      updateGroup(const GroupInfo& group) throw (DmException);
    void      deleteGroup(const std::string& groupName) throw (DmException);

    UserInfo newUser(const std::string& uname) throw (DmException);
    UserInfo getUser(const std::string& userName) throw (DmException);
    UserInfo getUser(const std::string& key,
                     const boost::any& value) throw (DmException);
    void     updateUser(const UserInfo& user) throw (DmException);
    void     deleteUser(const std::string& userName) throw (DmException);
    
    std::vector<GroupInfo> getGroups(void) throw (DmException);
    std::vector<UserInfo>  getUsers (void) throw (DmException);

    void getIdMap(const std::string& userName,
                  const std::vector<std::string>& groupNames,
                  UserInfo* user,
                  std::vector<GroupInfo>* groups) throw (DmException);

   protected:
    /// The Oracle connection pool.
    oracle::occi::ConnectionPool* pool_;

    /// The Oracle connection
    oracle::occi::Connection* conn_;

    /// Mapfile
    std::string mapFile_;
    
    /// Map host DN to root.
    bool hostDnIsRoot_;
  };
  
};

#endif // AUTHNORACLE_H
