/// @file   Queries.h
/// @brief  Oracle queries.
/// @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
#ifndef QUERIES_H
#define	QUERIES_H

#include <dmlite/cpp/exceptions.h>
#include <occi.h>

extern const char* STMT_GET_FILE_BY_ID;
extern const char* STMT_GET_FILE_BY_GUID;
extern const char* STMT_GET_FILE_BY_NAME;
extern const char* STMT_GET_LIST_FILES;
extern const char* STMT_GET_SYMLINK;
extern const char* STMT_GET_FILE_REPLICAS;
extern const char* STMT_GET_REPLICA_BY_URL;
extern const char* STMT_GET_REPLICA_BY_ID;
extern const char* STMT_GET_COMMENT;
extern const char* STMT_SET_GUID;
extern const char* STMT_SET_COMMENT;
extern const char* STMT_INSERT_COMMENT;
extern const char* STMT_INSERT_FILE;
extern const char* STMT_INSERT_SYMLINK;
extern const char* STMT_DELETE_FILE;
extern const char* STMT_DELETE_COMMENT;
extern const char* STMT_DELETE_SYMLINK;
extern const char* STMT_SELECT_NLINK_FOR_UPDATE;
extern const char* STMT_UPDATE_NLINK;
extern const char* STMT_UPDATE_PERMS;
extern const char* STMT_DELETE_REPLICA;
extern const char* STMT_ADD_REPLICA;
extern const char* STMT_TRUNCATE_FILE;
extern const char* STMT_CHANGE_NAME;
extern const char* STMT_CHANGE_PARENT;
extern const char* STMT_UTIME;
extern const char* STMT_UPDATE_REPLICA;

// Authn

extern const char* STMT_GET_ALL_USERS;
extern const char* STMT_GET_USERINFO_BY_NAME;
extern const char* STMT_GET_USERINFO_BY_UID;
extern const char* STMT_GET_ALL_GROUPS;
extern const char* STMT_GET_GROUPINFO_BY_NAME;
extern const char* STMT_GET_GROUPINFO_BY_GID;
extern const char* STMT_GET_UNIQ_UID_FOR_UPDATE;
extern const char* STMT_GET_UNIQ_GID_FOR_UPDATE;
extern const char* STMT_UPDATE_UNIQ_UID;
extern const char* STMT_UPDATE_UNIQ_GID;
extern const char* STMT_INSERT_UNIQ_UID;
extern const char* STMT_INSERT_UNIQ_GID;
extern const char* STMT_INSERT_USER;
extern const char* STMT_INSERT_GROUP;
extern const char* STMT_UPDATE_USER;
extern const char* STMT_UPDATE_GROUP;
extern const char* STMT_DELETE_USER;
extern const char* STMT_DELETE_GROUP;

/// Get a preparted statement
inline oracle::occi::Statement* getPreparedStatement(oracle::occi::Connection* conn,
                                                     const char* query)
{
  oracle::occi::Statement* stmt;

  try {
    stmt = conn->createStatement(query);
    // Make sure autocommit is always disabled
    stmt->setAutoCommit(false);
  }
  catch (oracle::occi::SQLException& ex) {
    throw dmlite::DmException(DM_QUERY_FAILED, std::string("Prepare: ") + ex.getMessage());
  }
  return stmt;
}

#endif // QUERIES_H
